@@ -1,4 +1,5 @@
-import py, os, cffi, re
+import os, cffi, re
+import pytest
 import _cffi_backend
 
 
@@ -7,7 +8,7 @@ def getlines():
         f = open(os.path.join(os.path.dirname(cffi.__file__),
                               '..', 'c', 'commontypes.c'))
     except IOError:
-        py.test.skip("cannot find ../c/commontypes.c")
+        pytest.skip("cannot find ../c/commontypes.c")
     lines = [line for line in f.readlines() if line.strip().startswith('EQ(')]
     f.close()
     return lines
