@@ -95,6 +95,8 @@ def test_large_constant():
     assert ffi.integer_const('BIGPOS') == 420000000000
     assert ffi.integer_const('BIGNEG') == -420000000000
 
+@pytest.mark.thread_unsafe(
+    reason="Worker threads might call dlclose concurrently")
 def test_function():
     import _cffi_backend
     from re_python_pysrc import ffi
@@ -102,6 +104,8 @@ def test_function():
     assert lib.add42(-10) == 32
     assert type(lib.add42) is _cffi_backend.FFI.CData
 
+@pytest.mark.thread_unsafe(
+    reason="Worker threads might call dlclose concurrently")
 def test_function_with_varargs():
     import _cffi_backend
     from re_python_pysrc import ffi
@@ -121,6 +125,8 @@ def test_dlopen_none():
     lib = ffi.dlopen(name)
     assert lib.strlen(b"hello") == 5
 
+@pytest.mark.thread_unsafe(
+    reason="Worker threads might call dlclose concurrently")
 def test_dlclose():
     import _cffi_backend
     from re_python_pysrc import ffi
@@ -135,6 +141,8 @@ def test_dlclose():
         "library '%s' has been closed" % (str_extmod,))
     ffi.dlclose(lib)   # does not raise
 
+@pytest.mark.thread_unsafe(
+    reason="Worker threads might call dlclose concurrently")
 def test_constant_via_lib():
     from re_python_pysrc import ffi
     lib = ffi.dlopen(extmod)
@@ -160,6 +168,7 @@ def test_enum():
     e = ffi.cast("enum foo_e", 2)
     assert ffi.string(e) == "CC"
 
+@pytest.mark.thread_unsafe(reason="tests would share a compilation directory")
 def test_include_1():
     sub_ffi = FFI()
     sub_ffi.cdef("static const int k2 = 121212;")
@@ -185,6 +194,7 @@ def test_include_1():
     p = ffi.new("bar_t *", [5, b"foobar"])
     assert p.a[4] == ord('a')
 
+@pytest.mark.thread_unsafe(reason="mutates shared global state")
 def test_global_var():
     from re_python_pysrc import ffi
     lib = ffi.dlopen(extmod)
@@ -195,12 +205,16 @@ def test_global_var():
     p[0] -= 1
     assert lib.globalvar42 == 1238
 
+@pytest.mark.thread_unsafe(
+    reason="Worker threads might call dlclose concurrently")
 def test_global_const_int():
     from re_python_pysrc import ffi
     lib = ffi.dlopen(extmod)
     assert lib.globalconst42 == 4321
     pytest.raises(AttributeError, ffi.addressof, lib, 'globalconst42')
 
+@pytest.mark.thread_unsafe(
+    reason="Worker threads might call dlclose concurrently")
 def test_global_const_nonint():
     from re_python_pysrc import ffi
     lib = ffi.dlopen(extmod)
@@ -213,6 +227,8 @@ def test_rtld_constants():
     ffi.RTLD_LAZY
     ffi.RTLD_GLOBAL
 
+@pytest.mark.thread_unsafe(
+    reason="Worker threads might call dlclose concurrently")
 def test_no_such_function_or_global_var():
     from re_python_pysrc import ffi
     lib = ffi.dlopen(extmod)
@@ -266,10 +282,12 @@ def test_selfref():
     from re_python_pysrc import ffi
     ffi.new("selfref_ptr_t")
 
+@pytest.mark.thread_unsafe(
+    reason="Worker threads might call dlclose concurrently")
 def test_dlopen_handle():
     import _cffi_backend
     from re_python_pysrc import ffi
-    if sys.platform == 'win32' or is_musl:
+    if sys.platform == 'win32' or is_musl or sys.platform.startswith('freebsd'):
         pytest.skip("uses 'dl' explicitly")
     ffi1 = FFI()
     ffi1.cdef("""void *dlopen(const char *filename, int flags);
@@ -286,3 +304,35 @@ def test_dlopen_handle():
 
     err = lib1.dlclose(handle)
     assert err == 0
+
+@pytest.mark.thread_unsafe(reason="workers would share a compilation directory")
+def test_rec_structs_1():
+    ffi = FFI()
+    ffi.cdef("struct B { struct C* c; }; struct C { struct B b; };")
+    ffi.set_source('test_rec_structs_1', None)
+    ffi.emit_python_code(str(tmpdir.join('_rec_structs_1.py')))
+    #
+    if sys.version_info[:2] >= (3, 3):
+        import importlib
+        importlib.invalidate_caches()  # issue 197, maybe
+    #
+    from _rec_structs_1 import ffi
+    # the following line used to raise TypeError
+    # unless preceeded by 'ffi.sizeof("struct C")'.
+    sz = ffi.sizeof("struct B")
+    assert sz == ffi.sizeof("int *")
+
+@pytest.mark.thread_unsafe(reason="workers would share a compilation directory")
+def test_rec_structs_2():
+    ffi = FFI()
+    ffi.cdef("struct B { struct C* c; }; struct C { struct B b; };")
+    ffi.set_source('test_rec_structs_2', None)
+    ffi.emit_python_code(str(tmpdir.join('_rec_structs_2.py')))
+    #
+    if sys.version_info[:2] >= (3, 3):
+        import importlib
+        importlib.invalidate_caches()  # issue 197, maybe
+    #
+    from _rec_structs_2 import ffi
+    sz = ffi.sizeof("struct C")
+    assert sz == ffi.sizeof("int *")
