@@ -1,7 +1,8 @@
 import sys, os
+from cffi._imp_emulation import load_dynamic
 
 if sys.version_info < (3,):
-    __all__ = ['u', 'arraytostring']
+    __all__ = ['u', 'arraytostring', 'load_dynamic']
 
     class U(object):
         def __add__(self, other):
@@ -16,7 +17,7 @@ if sys.version_info < (3,):
         return a.tostring()
 
 else:
-    __all__ = ['u', 'unicode', 'long', 'arraytostring']
+    __all__ = ['u', 'unicode', 'long', 'arraytostring', 'load_dynamic']
     u = ""
     unicode = str
     long = int
@@ -33,9 +34,14 @@ class StdErrCapture(object):
             from io import StringIO
         self.old_stderr = sys.stderr
         sys.stderr = f = StringIO()
+        if hasattr(sys, '__unraisablehook__'):           # work around pytest
+            self.old_unraisablebook = sys.unraisablehook # on recent CPythons
+            sys.unraisablehook = sys.__unraisablehook__
         return f
     def __exit__(self, *args):
         sys.stderr = self.old_stderr
+        if hasattr(self, 'old_unraisablebook'):
+            sys.unraisablehook = self.old_unraisablebook
 
 
 class FdWriteCapture(object):
@@ -44,8 +50,8 @@ class FdWriteCapture(object):
 
     def __init__(self, capture_fd=2):    # stderr by default
         if sys.platform == 'win32':
-            import py
-            py.test.skip("seems not to work, too bad")
+            import pytest
+            pytest.skip("seems not to work, too bad")
         self.capture_fd = capture_fd
 
     def __enter__(self):
@@ -67,14 +73,13 @@ class FdWriteCapture(object):
         return self._value
 
 def _verify(ffi, module_name, preamble, *args, **kwds):
-    import imp
     from cffi.recompiler import recompile
     from .udir import udir
     assert module_name not in sys.modules, "module name conflict: %r" % (
         module_name,)
     kwds.setdefault('tmpdir', str(udir))
     outputfilename = recompile(ffi, module_name, preamble, *args, **kwds)
-    module = imp.load_dynamic(module_name, outputfilename)
+    module = load_dynamic(module_name, outputfilename)
     #
     # hack hack hack: copy all *bound methods* from module.ffi back to the
     # ffi instance.  Then calls like ffi.new() will invoke module.ffi.new().
@@ -112,3 +117,21 @@ else:
         extra_compile_args = ['-Werror', '-Wall', '-Wextra', '-Wconversion',
                               '-Wno-unused-parameter',
                               '-Wno-unreachable-code']
+
+is_musl = False
+if sys.platform == 'linux':
+    try:
+        from packaging.tags import platform_tags
+    except ImportError:
+        pass
+    else:
+        tagset = frozenset(platform_tags())
+        is_musl = any(t.startswith('musllinux') for t in tagset)
+        if is_musl and sys.version_info >= (3, 12):
+            if any(t.startswith('musllinux_1_1_') for t in tagset) and not any(t.startswith('musllinux_1_2_') for t in tagset):
+                # gcc 9.2.0 in the musllinux_1_1 build container has a bug in its sign-conversion warning detection that
+                # bombs on the definition of _PyLong_CompactValue under Python 3.12; disable warnings-as-errors for that
+                # specific error on musl 1.1
+                extra_compile_args.append('-Wno-error=sign-conversion')
+
+        del platform_tags
